import { Type } from '@angular/core';
import { Action, combineReducers, Selector, createSelector } from '@ngrx/store';

import * as ChatStore from './chat';
import * as StatusStore from './status';
import * as SyncStore from './sync';

export interface State {
  chat: ChatStore.State;
  status: StatusStore.State;
  sync: SyncStore.State;
}

export const effects: Type<any>[] = [
  ChatStore.Effects,
  StatusStore.Effects,
  SyncStore.Effects
];

export function reducers(state: State | undefined, action: Action) {
  return combineReducers({
    chat: ChatStore.reducer,
    status: StatusStore.reducer,
    sync: SyncStore.reducer
  })(state, action);
}

export function selectors<S>(selector: Selector<any, State>) {
  return {
    ChatSelector: ChatStore.selectors(
      createSelector(
        selector,
        (state: State) => state.chat
      )
    ),
    StatusSelector: StatusStore.selectors(
      createSelector(
        selector,
        (state: State) => state.status
      )
    ),
    SyncSelector: SyncStore.selectors(
      createSelector(
        selector,
        (state: State) => state.sync
      )
    )
  };
}
